#ifndef WIT_H
#define WIT_H

namespace wit {

/**
 * Used to select which application features to enable. It is strongly recommended to only enable the
 * features that the application will use to reduce event processing overhead.
 */
enum AppFeatures {
    NoFeatures = 0,         /**< Do not enable any features. */
    UseWiiRemote = 1,       /**< Enable the use of Wii Remotes. */
    UseGameCube = 2,        /**< Enable the use of GameCube controllers. */
    UseSound = 4,           /**< Enable the use of sound. */
    UseFAT = 8,             /**< Enable the use of a FAT filesystem on the SD card. */
    UseGX = 16,             /**< Enable the GX rendering engine. */
    UseConsole = 32,        /**< Enable the text console. (Note, incompatible with UseDoubleBuffer; see \ref wit::Console) */
    UseJoysticks = 64,      /**< Enable tracking the analog joysticks on Nunchuks, Classic Controllers, and GameCube controllers. */
    UsePointer = 128,       /**< Enable tracking the infrared sensor on Wii Remotes. */
    UseMotion = 256,        /**< Enable tracking the motion sensor on Wii Remotes and Nunchuks. */
    UseDoubleBuffer = 512,  /**< Enable double buffering. */
    AllFeatures =           /**< Enable all features. */
        UseWiiRemote | UseGameCube | UseSound | UseFAT | UseGX | UseConsole | UseJoysticks | UsePointer | UseMotion | UseDoubleBuffer
};

enum ShutdownMode {
    Shutdown_Auto,       /**< Automatically use Shutdown_Idle or Shutdown_Standby based on the Wii system settings. */
    Shutdown_Idle,       /**< Put the Wii in idle mode, with WiiConnect24 disabled (red LED). */
    Shutdown_Standby     /**< Put the Wii in standby mode, with WiiConnect24 enabled (yellow LED). */
};

enum Button {
    Button_None = 0,    /**< No button */ 
    Button_Up,          /**< Up arrow on the D-pad, or strumming up on the Guitar */
    Button_Down,        /**< Down arrow on the D-pad, or strumming down on the Guitar */
    Button_Left,        /**< Left arrow on the D-pad */
    Button_Right,       /**< Right arrow on the D-pad */
    Button_1,           /**< 1 button on the Wii Remote */
    Button_2,           /**< 2 button on the Wii Remote */
    Button_A,           /**< A button on the Wii Remote, Classic Controller, or GameCube controller */
    Button_B,           /**< B button on the Wii Remote, Classic Controller, or GameCube controller */
    Button_C,           /**< C button on the Nunchuk */
    Button_X,           /**< X button on the Classic Controller or GameCube controller */
    Button_Y,           /**< Y button on the Classic Controller or GameCube controller */
    Button_Z,           /**< Z button on the Nunchuk or GameCube controller */
    Button_ZL,          /**< ZL button on the Classic Controller */
    Button_ZR,          /**< ZR button on the Classic Controller */
    Button_LClick,      /**< Fully depressed L button on the Classic Controller or GameCube controller */
    Button_RClick,      /**< Fully depressed R button on the Classic Controller or GameCube controller */
    Button_Minus,       /**< Minus button on the Wii Remote, Classic Controller, or Guitar */
    Button_Plus,        /**< Plus button on the Wii Remote, Classic Controller, or Guitar */
    Button_Home,        /**< Home button on the Wii Remote or Classic Controller */
    Button_Start,       /**< Start button on the GameCube controller */
    Button_Max,         /**< A value greater than all button values */
    Button_Yellow = Button_1,   /**< Yellow button on the Guitar */
    Button_Green  = Button_2,   /**< Green button on the Guitar */
    Button_Blue   = Button_A,   /**< Blue button on the Guitar */
    Button_Red    = Button_B,   /**< Red button on the Guitar */
    Button_Orange = Button_C    /**< Orange button on the Guitar */
};

enum Controller {
    WiiRemote,      /**< The event was generated by a Wii Remote. */
    Nunchuk,        /**< The event was generated by a Nunchuk. */
    Classic,        /**< The event was generated by a Classic Controller. */
    Guitar,         /**< The event was generated by a Guitar Hero 3 controller. */
    BalanceBoard,   /**< The event was generated by a Balance Board. */
    GameCube        /**< The event was generated by a GameCube controller. */
};

enum AnalogButton {
    Analog_Left,     /**< L button on the Classic Controller or GameCube controller */
    Analog_Right,    /**< R button on the Classic Controller or GameCube controller */
    Analog_A,        /**< A button on the GameCube controller */
    Analog_B,        /**< B button on the GameCube controller */
    Analog_Whammy = Analog_Left   /**< Whammy bar on the Guitar */
};


/**
 * Returns the absolute value of the parameter. Supports any numeric type.
 *
 * \global
 */
template <typename T>
inline T witAbs(T n) { return (n < 0) ? -n : n; }

}

using wit::witAbs;

#endif
